/* name:    BSF_UTIL.CLS
   authors: Lee Peedin, Rony G. Flatscher
   date:    2006-03-09
   version: 0.5
   license: CPL 1.0, APL 2.0: (see below)

   defines class "BSF.DIALOG" for easy creating message, option and input dialogs

--------

class method messageBox(...): shows dialog, returns always .nil

   arguments:
      message
      message, title
      message, [title], messageType

--------

class method dialogBox(...): shows dialog, returns the number of the push button pressed (0=first key, ...)
                       or -1, if user closed dialog with other means
   arguments:
      message
      message, title
      message, [title], messageType
      message, [title], [messageType], optionType
      message, [title], [messageType], [optionType], [icon], textOfButtons[, defaultButton]

--------

class method inputBox(...): shows dialog, returns the entered/chosen value or .nil if user closed dialog
                      with other means

   arguments:
      message
      message, defaultValue
      message, [title], messageType
      message, [title], messageType, [icon], textOfOptions[, defaultValue]

--------

where messageType is one of:

      error                   - includes X icon
      information             - includes i icon
      plain                   - no icon included
      question                - includes ? mark icon
      warning                 - includes ! icon

where optionType is one of:

      OkCancel                - includes OK/Cancel buttons
      YesNo                   - includes Yes/No buttons
      YesNoCancel             - includes Yes/No/Cancel buttons
      default                 - includes OK/Cancel, uses messageType "question"


*/

/*


showConfirmDialog
    Used to require confirmation from the user
    Icons vary based on the messageType parameter
    Buttons vary based on the optionType parameter
    Return a value associated with the button selected or the dialog CLOSE button
    Displays in the middle of the users screen

showInputDialog
    Allows the user to enter a single value OR select from a pre-defined array of values
    Icons vary based on the messageType parameter
    Has 2 buttons (OK & Cancel)
    Return the value entered or selected by the user (.nil if Cancel or CLOSE)
    Displays in the middle of the users screen


    Valid messageType parameter

      error                   - includes X icon
      information             - includes i icon
      warning                 - includes ! icon
      question                - includes ? mark icon
      plain                   - no icon included

    Valid optionType parameter
      default                 - includes OK/Cancel, questionType
      OkCancel                - includes OK/Cancel buttons
      YesNo                   - includes Yes/No buttons
      YesNoCancel             - includes Yes/No/Cancel buttons

Button Return Values    -1  Dialog CLOSE
                         0  Yes or OK depending on the optionType
                         1  No
                         2  Cancel

------------------------ Apache Version 2.0 license -------------------------
   Copyright 2006 Rony G. Flatscher

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
-----------------------------------------------------------------------------



------------------------ Common Public License 1.0 --------------------------
   Copyright 2006 Rony G. Flatschre

   This program and the accompanying materials are made available under the
   terms of the Common Public License v1.0 which accompanies this distribution.

   It may also be viewed at: http://www.opensource.org/licenses/cpl1.0.php
-----------------------------------------------------------------------------


*/

::requires BSF.CLS      -- get the Java support


::class bsf.dialog public

::method init class
  expose constants javaDialogClass
   -- initialize attributes to .nil
  constants      =.nil
  javaDialogClass=.nil


::method javaDialogClass class   -- the Java class object proxy for creating dialogs
  expose javaDialogClass

  if .nil=javaDialogClass then   -- not yet defined, get reference to it
     javaDialogClass=.bsf~bsf.import('javax.swing.JOptionPane', .nil)

  return javaDialogClass


::method constants class
  expose constants

  if .nil=constants then     -- not yet defined, create it
  do
       /* initialisation needed for this module */
          -- get easy access to constants
     o =  bsf.wrapStaticFields('javax.swing.JOptionPane')
          -- message types
     constants = .directory~new
     constants~error                  = o~error_message
     constants~error.title            = "Error"

     constants~information            = o~information_message
     constants~information.title      = "Information"

     constants~warning                = o~warning_message
     constants~warning.title          = "Warning"

     constants~question               = o~question_message
     constants~question.title         = "Question"

     constants~plain                  = o~plain_message
     constants~plain.title            = ""

       -- dialog option types
     constants~default                = o~default_option
     constants~default.buttonText     = "OK Cancel"

     constants~OkCancel               = o~ok_cancel_option
     constants~OkCancel.buttonText    = "OK Cancel"

     constants~YesNo                  = o~yes_no_option
     constants~YesNo.buttonText       = "Yes No"

     constants~YesNoCancel            = o~yes_no_cancel_option
     constants~YesNoCancel.buttonText = "Yes No Cancel"
  end

  return constants


   -- method to make it simple to invoke a dialog with push buttons as options
   -- implementation uses javax.swing.JOptionPane
::method dialogBox class
  use arg message, title, messageType, optionType, icon, textOfButtons, defaultButton

  constants      =self~constants          -- get constants
  javaDialogClass=self~javaDialogClass    -- get Java class object

  if arg()=1 then       -- only one argument given
  do
     return javaDialogClass~new~showConfirmDialog(.nil, message)
  end
  else
  do
     -- messageType
     swingMessageType=constants~entry(messageType) -- get the Java encoding of message type
     if .nil=swingMessageType then
     do
         messageType="plain"
         swingMessageType=constants~plain   -- default messageType
     end

     -- title omitted, get default title
     if arg(2, 'Omitted') then   -- title not given, use a default one
     do
        title=constants~entry(messageType~strip".TITLE")
        if title="" then title="Message"  -- plain, but indicate that this is a message!
     end

     -- optionType
     swingOptionType=constants~entry(optionType)
     if .nil=swingOptionType then
     do
        optionType="default"
        swingOptionType=constants~default   -- default optionType
     end

     if arg()<5 then    -- no more arguments (strings for buttons, and setting default button)
     do
        return javaDialogClass~new~showConfirmDialog(.nil,message,title,swingOptionType,swingMessageType)
     end

     -- optional icon
     if arg(5, 'Omitted') then
        icon=.nil

     -- optional button text collection and optional default button
     if arg(6, 'Omitted') then
        textOfButtons=constants~entry(optionType~space".ButtonText")~makearray(' ')

     javaArr=bsf.createArray(.bsf4rexx~object.class, textOfButtons~items)  -- create Java array of needed size
     i=0
     do entry over textOfButtons -- this way any ooRexx collection object can be used !
        i=i+1
        javaArr[i]=entry
     end

     if arg(7, 'Omitted') then   -- no default button
     do
        defaultButton=.nil
     end

     return javaDialogClass~new~showOptionDialog(.nil,message,title,swingOptionType,swingMessageType, icon, javaArr, defaultButton)
  end
  return -99  -- return impossible value



/*
   -- method to make it simple to invoke a dialog with input field or drop-down list to choose from
   -- implementation uses javax.swing.JOptionPane

   test ".bsfdialog~inputBox(...):'
       (message                                                             )
       (message, defaultValue                                               )
       (message, [title], messageType                                       )
       (message, [title], messageType, [icon], textOfOptions[, defaultValue])

        ... method returns the entered/chosen value or .nil (user closed dialog)
 */
::method inputBox class
  nrArgs=arg()          -- get number of arguments

  constants      =self~constants          -- get constants
  javaDialogClass=self~javaDialogClass    -- get Java class object

  if nrArgs=1 then       -- only one argument given
  do
     use arg message
     return javaDialogClass~new~showInputDialog(.nil, message)
  end
  else if nrArgs=2 then
  do
     use arg message, defaultValue
     return javaDialogClass~new~showInputDialog(.nil, message, defaultValue)
  end


  use arg message, title, messageType, icon, textOfOptions, defaultValue
  -- messageType
  swingMessageType=constants~entry(messageType) -- get the Java encoding of message type
  if .nil=swingMessageType then
  do
      messageType="plain"
      swingMessageType=constants~plain   -- default messageType
  end

  -- title omitted, get default title
  if arg(2, 'Omitted') then   -- title not given, use a default one
  do
     title=constants~entry(messageType~strip".TITLE")
     if title="" then title="Message"  -- plain, but indicate that this is a message!
  end


  if nrArgs=3 then
  do
     return javaDialogClass~new~showInputDialog(.nil, message, title, swingMessageType)
  end

  -- optional icon
  if arg(4, 'Omitted') then
     icon=.nil

  -- option collection
  if arg(5)~hasmethod("PUT")=.false then -- not a collection ?
  do
     textOfOptions=space(textOfOptions)~makearray(' ')
  end

  -- optional button text collection and optional default button
  javaArr=bsf.createArray(.bsf4rexx~object.class, textOfOptions~items)  -- create Java array of needed size
  i=0
  do entry over textOfOptions -- this way any ooRexx collection object can be used !
     i=i+1
     javaArr[i]=entry
  end

  if arg(6, 'Omitted') then   -- no default button
  do
     defaultValue=.nil
  end

  return javaDialogClass~new~showInputDialog(.nil,message,title,swingMessageType, icon, javaArr, defaultValue)


   -- class method to make it simple to invoke the desired functionality
   -- implementation uses javax.swing.JOptionPane
::method messageBox class
  use arg message, title, messageType

  constants      =self~constants          -- get constants
  javaDialogClass=self~javaDialogClass    -- get Java class object

  if arg()=1 then       -- only one argument given
  do
     javaDialogClass~new~showMessageDialog(.nil, message)
  end
  else
  do
     -- default to plain message box
     if constants~hasentry(messageType)=.false then
     do
        messageType="plain"
     end

     swingMessageType=constants~entry(messageType) -- get the Java encoding of message type
     -- title omitted, get default title
     if arg(2, 'Omitted') then   -- title not given, use a default one
     do
        title=constants~entry(messageType~strip".TITLE")
        if title="" then title="Message"  -- plain, but indicate that this is a message!
     end

     javaDialogClass~new~showMessageDialog(.nil, message, title, swingMessageType)
  end
  return .nil     -- message box returns always .nil




